// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

#include "FLTNOSService.h"

#include "nim_cpp_wrapper/api/nim_cpp_nos.h"

FLTNOSService::FLTNOSService() {
    m_serviceName = "NOSService";
}

void FLTNOSService::onMethodCalled(const std::string& method, const flutter::EncodableMap* arguments,
                                   std::shared_ptr<flutter::MethodResult<flutter::EncodableValue>> result) {
    if (method == "upload") {
        upload(arguments, result);
    } else {
        result->NotImplemented();
    }
}

void FLTNOSService::upload(const flutter::EncodableMap* arguments, std::shared_ptr<flutter::MethodResult<flutter::EncodableValue>> result) {
    if (!arguments) {
        return;
    }

    std::string filePath = "";
    auto iter = arguments->find(flutter::EncodableValue("filePath"));
    if (iter != arguments->end()) {
        if (iter->second.IsNull()) {
            result->Error("", "", NimResult::getErrorResult(-1, "upload params error"));
            return;
        }
        filePath = std::get<std::string>(iter->second);
    }

    std::string sceneKey = "";
    iter = arguments->find(flutter::EncodableValue("sceneKey"));
    if (iter != arguments->end()) {
        if (!iter->second.IsNull()) {
            sceneKey = std::get<std::string>(iter->second);
        }
    }

    bool res = nim::NOS::UploadResource2(filePath, sceneKey, std::bind(&FLTNOSService::uploadMediaCallback, this, std::placeholders::_1, std::placeholders::_2),
                              std::bind(&FLTNOSService::progressCallback, this, std::placeholders::_1, std::placeholders::_2));

    if(res) {
        result->Success(NimResult::getSuccessResult());
    }else {
        result->Error("","",NimResult::getErrorResult(-1, "nos upload failed!"));
    }
}

void FLTNOSService::uploadMediaCallback(nim::NIMResCode res_code, const std::string& url) {
    if (res_code == nim::kNIMResSuccess) {
        flutter::EncodableMap arguments;
        arguments.insert(std::make_pair("url", url));
        arguments.insert(std::make_pair("status", "transferred"));
        arguments.insert(std::make_pair("transferType", "upload"));
        notifyEvent("onNOSTransferStatus", arguments);
    }
}

void FLTNOSService::progressCallback(int64_t completed_size, int64_t file_size) {
    flutter::EncodableMap arguments;
    double progress = (double)completed_size / file_size;
    arguments.insert(std::make_pair("progress", progress));
    notifyEvent("onNOSTransferProgress", arguments);
}

void FLTNOSService::speedCallback(int64_t speed) {
}

void FLTNOSService::transferInfoCallback(int64_t actual_size, int64_t speed) {
}
